#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['RMVtransport']

package_data = \
{'': ['*']}

install_requires = \
['async-timeout', 'lxml', 'httpx']

extras_require = \
{'test': ['pytest >= 5.2.2',
          'pytest-cov >= 2.8.1',
          'pytest-asyncio',
          'pylint',
          'tox',
          'pytest-mock',
          'pytest-httpx',
          'asynctest',
          'codecov',
          'coverage',
          'mock',
          'flake8',
          'mypy',
          'bandit',
          'pre-commit',
          'black']}

setup(name='PyRMVtransport',
      version='0.3.1',
      description='Python library to make use of transport information from opendata.rmv.de.',
      author='cgtobi',
      author_email='cgtobi@gmail.com',
      url='https://github.com/cgtobi/PyRMVtransport',
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      extras_require=extras_require,
      python_requires='>=3.6',
     )
